from yta_editor_time.specifications.resolved.frame import FrameIndexSpecificationResolved


__all__ = [
    'FrameIndexSpecificationResolved'
]


"""
A full flow resolving:

# Datos del clip
fps = Fraction(60)
total_frames = FrameCount(150)

# Contexto actual (frame 50)
context = EvaluationContext(
    fps=fps,
    total_frames=total_frames,
    frame_index=FrameIndex(50)
)

# “Aplica del frame 40 al 80”
spec = FrameIndexSpecification(
    start=FrameIndex(40),
    end=FrameIndex(80),
    length=None
)

# Resolución (una sola vez)
resolved = resolve_frame_index_spec(spec, total_frames)

# Evaluación por frame
active = is_active(context, resolved)

print(active)  # True
"""


from yta_editor_time.enum import SpecificationRequirement
from yta_editor_time.specifications.types import TimeMoment, TimeDuration
from dataclasses import dataclass
from typing import Union

    
@dataclass(frozen = True)
class TimeSpecification:
    """
    The time range of the element that must be affected
    according to this specification.

    TODO: Explain like the others
    """

    # TODO: By now we are not implementing the 'anchor'
    # because it complicates the logic a lot...
    # anchor: Anchor
    # """
    # The position, element or part of it to which this
    # specification is anchored.
    # """
    start_t: Union[TimeMoment, None]
    """
    The first time moment that must be affected by this
    specification.
    """
    end_t: Union[TimeMoment, None]
    """
    The last time moment that must be affected by this
    specification.
    """
    duration: Union[TimeDuration, None]
    """
    The total duration that this specification must
    last.
    """

    @property
    def requirements(
        self
    ) -> dict[SpecificationRequirement]:
        """
        The requirements of this specification that are
        needed to be able to resolve it.
        """
        return {
            SpecificationRequirement.FPS
        }

    def __post_init__(
        self
    ):
        # Normalize enums
        # object.__setattr__(self, 'anchor', Anchor.to_enum(self.anchor))

        # exactly two of three must be provided
        provided = sum(
            x is not None
            for x in (self.start_t, self.end_t, self.duration)
        )

        if provided != 2:
            raise ValueError(
                'Provide exactly two of: start_t, end_t, duration'
            )

        if (
            self.start_t and
            self.end_t
        ):
            if self.end_t.value < self.start_t.value:
                raise ValueError('end_t must be >= start_t')

        if (
            self.duration and
            self.duration.value == 0
        ):
            raise ValueError('duration must be > 0')


from dataclasses import dataclass
from typing import Union


# Frames
@dataclass(frozen = True)
class FrameIndex:
    """
    *Dataclass*

    Dataclass to represent a the index of a frame.
    """
    value: int
    """
    The index of the frame.
    """

    def __post_init__(
        self
    ):
        if self.value < 0:
            raise ValueError('FrameIndex must be >= 0')
        
    def __int__(
        self
    ) -> int:
        return self.value
        
@dataclass(frozen = True)
class FramesNumber:
    """
    The total amount of frames.
    """
    value: int
    """
    The amount of frames.
    """

    def __post_init__(
        self
    ):
        if self.value < 0:
            raise ValueError('FramesNumber must be >= 0')
        
    def __int__(
        self
    ) -> int:
        return self.value
    
# Time
@dataclass(frozen = True)
class TimeMoment:
    """
    *Dataclass*

    Dataclass to represent a time moment.
    """
    value: Union[float, 'Fraction']
    """
    The time moment.
    """

    def __post_init__(
        self
    ):
        if self.value < 0:
            raise ValueError('TimeMoment must be >= 0')
        
    def __float__(
        self
    ) -> float:
        return float(self.value)
        
@dataclass(frozen = True)
class TimeDuration:
    """
    *Dataclass*

    Dataclass to represent a total time duration (in
    seconds).
    """
    value: Union[float, 'Fraction']
    """
    The total duration (in seconds).
    """

    def __post_init__(
        self
    ):
        if self.value < 0:
            raise ValueError('TimeDuration must be >= 0')
        
    def __float__(
        self
    ) -> float:
        return float(self.value)
    
@dataclass(frozen = True)
class Fps:
    """
    *Dataclass*

    Dataclass to represent an amount of frames per
    second.
    """
    value: Union[float, 'Fraction']
    """
    The amount of frames per second.
    """

    def __post_init__(
        self
    ):
        if self.value < 0:
            raise ValueError('Fps must be >= 0')
        
    def __float__(
        self
    ) -> float:
        return float(self.value)
        
# Progress
@dataclass(frozen = True)
class ProgressValue:
    """
    *Dataclass*

    Dataclass to represent a progress value, that will
    be always in the `[0.0, 1.0]` range.
    """
    value: Union[float, 'Fraction']
    """
    The progress value in the `[0.0, 1.0]` range.
    """

    def __post_init__(
        self
    ):
        if (
            self.value < 0.0 or
            self.value > 1.0
        ):
            raise ValueError('ProgressValue must be in a [0.0, 1.0] range.')
        
    def __float__(
        self
    ) -> float:
        return float(self.value)